<?php

namespace Teknisa\PED\Controllers;

use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\ORM\DateTime;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\PED\Services\Ger02400CalendarioDeDiasNaoUteisService;

class Ger02400CalendarioDeDiasNaoUteisController {

	protected $dataSourceName;
    protected $dataSourceManager;
    protected $crudService;
    protected $entityManager;

    public function __construct(EntityManager $entityManager, $crudService, $dataSourceManager, Ger02400CalendarioDeDiasNaoUteisService $Ger02400CalendarioDeDiasNaoUteisService){
    	$this->entityManager       = $entityManager;
    	$this->crudService = $crudService;
    	$this->dataSourceManager = $dataSourceManager;
    	$this->Ger02400CalendarioDeDiasNaoUteisService    = $Ger02400CalendarioDeDiasNaoUteisService;
    }

    public function save($request, $response){
        $this->entityManager->getConnection()->beginTransaction();
        try{    
            $rows = $request->getDataSet()->getRows();
            $rowsToSave = array();
            foreach($rows as $key => $row){
                $calendar = $row['CALENDAR'];
                $condition = array(
                    array(
                        "columnName" =>'CDFILIAL',
                        "operator" => "=",
                        "value" => $row['CDFILIAL']
                    ),
                    array(
                        "columnName" =>$row['TABS'][$calendar]['DTFERI'],
                        "operator" => "=",
                        "value" => $row[$row['TABS'][$calendar]['DTFERI']]
                    )
                );
                $filterCriteria = new FilterCriteria($row['TABS'][$calendar]['TABFERI'], $condition);
                $dataSet = $this->dataSourceManager->findBy($filterCriteria);
                if( (empty($dataSet->getRows())) || (!$row['__is_new']) ){
                   array_push($rowsToSave, $rows[$key]);
                }
            }
            $newDataSet = new DataSet(null , $rowsToSave);
            $newRequest = new DTO\Request\DataSet($newDataSet, '', '', '');
            $rows = $this->crudService->getRowsToSave($newRequest,$rows[0]['TABS'][$rows[0]['CALENDAR']]['TABFERI']);
            $this->crudService->save($row['TABS'][$calendar]['TABFERI'], $rows);
            $this->entityManager->getConnection()->commit();
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
            $this->entityManager->rollback();
        } 
    }
    
    public function CalcCalendCompra($request, $response){
        $dataSet = $request->getDataset()->getRows();
        $adqPerFiliProd = $this->Ger02400CalendarioDeDiasNaoUteisService->adqPerFiliProd($dataSet);
        if($dataSet[1][0] == 12){
            $dtInicial = $dataSet[2][3];
            for($i=0; $i < count($adqPerFiliProd); $i++){
                if($adqPerFiliProd[$i]['IDTIPODIA'] == "R"){
                    $nrMeses= 12;
                    $adsRecorrente = $this->Ger02400CalendarioDeDiasNaoUteisService->adsRecorrente($dataSet, $adqPerFiliProd, $dtInicial, $nrMeses);
                } else {
                    $nrMeses = $dataSet[1][0];
                    $adsCalcEntregas = $this->Ger02400CalendarioDeDiasNaoUteisService->adsCalcEntregas($dataSet, $adqPerFiliProd, $dtInicial, $nrMeses);
                }
            }
        } else {
            $ano = DateUtil::getAno($dataSet[2][3]);
            for($j = 0; $j < count($dataSet[1][1]); $j++){    
                $mes = $dataSet[1][1][$j];
                if(strlen($mes) == 1){
                    $dtInicial = "01/0".$mes."/".$ano;
                } else if(strlen($mes) == 2){
                    $dtInicial = "01/".$mes."/".$ano;
                }
                for($i=0; $i < count($adqPerFiliProd); $i++){
                    if($adqPerFiliProd[$i]['IDTIPODIA'] == "R"){
                        $nrMeses = 12;
                        $adsRecorrente = $this->Ger02400CalendarioDeDiasNaoUteisService->adsRecorrente($dataSet, $adqPerFiliProd, $dtInicial, $nrMeses);
                    } else {
                        $nrMeses = $dataSet[1][0];
                        $adsCalcEntregas = $this->Ger02400CalendarioDeDiasNaoUteisService->adsCalcEntregas($dataSet, $adqPerFiliProd, $dtInicial, $nrMeses);
                    }
                }
            }
        }
    }
    
    public function verificaDia($request, $response){
        $dataSet =$request->getDataset()->getRows();
        $newDataSet = array();
        if(count($dataSet) > 1 ){
            for($i = 0; $i < count($dataSet); $i++){
                array_push($newDataSet, $this->verificaDiaFuncao($dataSet[$i][0]));
            }
            $response->addDataSet(new DataSet($dataSet[0][0]['TABCAL'],$newDataSet));
        }
        if(count($dataSet) == 1 ){
            $newDataSet = $this->verificaDiaFuncao($dataSet[0]);
            $response->addDataSet(new DataSet($dataSet[0]['TABCAL'],$newDataSet));
        }    
    }
    
    public function verificaDiaFuncao($currentRow){
        $newDataSet = array();
        $condition = array(
            array(
                "columnName" =>'CDFILIRECE',
                "operator" => "=",
                "value" => $currentRow['CDFILIAL']
            ),
            array(
                "columnName" =>'DTENTRSAID',
                "operator" => "=",
                "value" => $currentRow['DTDIA']
            ),
            array(
                "columnName" =>'NRORG',
                "operator" => "=",
                "value" => $currentRow['NRORG']
            )
        );
        $filterCriteria = new FilterCriteria('notafiscal', $condition);
        $dataSet = $this->dataSourceManager->findBy($filterCriteria);
        if (!empty($dataSet->getRows())){
            $newDataSet['ERRORMSG']=("Alteração bloqueada! Existe nota fiscal de entrada para o dia informado.");
        } else {
            $condition[0]['columnName']='CDFILIEMIT';
            $filterCriteria = new FilterCriteria('notafiscal', $condition);
            $dataSet = $this->dataSourceManager->findBy($filterCriteria);
            if (!empty($dataSet->getRows())){
                $newDataSet['ERRORMSG']=("Alteração bloqueada! Existe nota fiscal de saída para o dia informado.");
            } else{
                $condition = array(
                    array(
                        "columnName" =>'CDFILIAL',
                        "operator" => "=",
                        "value" => $currentRow['CDFILIAL']
                    ),
                    array(
                        "columnName" =>'NRORG',
                        "operator" => "=",
                        "value" => $currentRow['NRORG']
                    )

                );

                $filterCriteria = new FilterCriteria('paramplaneja', $condition);
                $dataSet = $this->dataSourceManager->findBy($filterCriteria);
                if(!empty($dataSet->getRows())) {    
                    if ($dataSet->getRows()[0]['IDCARDIAUTIL']=='S') {
                    $condition = array(
                        array(
                            "columnName" =>'CDFILIAL',
                            "operator" => "=",
                            "value" => $currentRow['CDFILIAL']
                        ),
                        array(
                            "columnName" =>'DTCARDAPIO',
                            "operator" => "=",
                            "value" => $currentRow['DTDIA']
                        ),
                        array(
                            "columnName" =>'NRORG',
                            "operator" => "=",
                            "value" => $currentRow['NRORG']
                        )
                    );
                    $filterCriteria = new FilterCriteria('cardadia_calendario', $condition);
                    $dataSet = $this->dataSourceManager->findBy($filterCriteria);
                    $rows = $dataSet->getRows();
                    foreach ($rows as $row) {
                        if ($row['NRRETIRADA']){
                            $newDataSet['ERRORMSG']=("Alteração bloqueada! Existe retirada para o dia informado.");
                        }
                    }
                    foreach ($rows as $row) {
                        if ($row['NRCARDAPIO']){
                            $DATADIA = DateTime::createFromFormat('d/m/Y H:i:s',$row['DTCARDAPIO']);
                            $DATAPROC = DateTime::createFromFormat('d/m/Y H:i:s',$currentRow['DTPROCE']);
                            if($DATADIA<$DATAPROC){
                                $newDataSet['ERRORMSG']=("Alteração bloqueada! Cardápio anterior à data de processamento não pode ser excluído." );
                                
                            } else {
                                $newDataSet['MSG']=("Atenção! Existe cardápio para o dia selecionado. Deseja excluí-lo?" );
                                $newDataSet['TYPE'] = "industrial";
                                $newDataSet['CDFILIAL'] = $currentRow['CDFILIAL'];
                                $newDataSet['DTCARDAPIO'] = $currentRow['DTDIA'];
                                $newDataSet['DTCARDIAPME'] = $currentRow['DTDIA'];
                                $newDataSet['DTPROCE'] =$currentRow['DTPROCE'];
                                $newDataSet['CDFILCENTPME'] = null;
                            }
                        }
                    }
                    $condition = array(
                        array(
                            "columnName" =>'CDFILIAL',
                            "operator" => "=",
                            "value" => $currentRow['CDFILIAL']
                        ),
                        array(
                            "columnName" =>'DTCARDIAPME',
                            "operator" => "=",
                            "value" => $currentRow['DTDIA']
                        ),
                        array(
                            "columnName" =>'NRORG',
                            "operator" => "=",
                            "value" => $currentRow['NRORG']
                        )
                    );
                    $filterCriteria = new FilterCriteria('cardiapme_calendario', $condition);
                    $dataSet = $this->dataSourceManager->findBy($filterCriteria);
                    $rows = $dataSet->getRows();
                    foreach ($rows as $row) {
                        if ($row['DTCARDIAPME']){
                            $DATADIA = DateTime::createFromFormat('d/m/Y H:i:s',$row['DTCARDIAPME']);
                            $DATAPROC = DateTime::createFromFormat('d/m/Y H:i:s',$currentRow['DTPROCE']);
                            if($DATADIA<$DATAPROC){ 
                                $newDataSet['ERRORMSG']=("Alteração bloqueada! Cardápio(Merenda) anterior à data de processamento não pode ser excluído." );
                                
                            } else {
                                $newDataSet['MSG']=("Atenção! Existe cardápio(Merenda) para o dia selecionado. Deseja excluí-lo?" );
                                $newDataSet['TYPE'] = "merenda";
                                $newDataSet['CDFILCENTPME'] = $row['CDFILCENTPME'];
                                $newDataSet['DTCARDAPIO'] = $currentRow['DTDIA'];
                                $newDataSet['DTCARDIAPME'] = $currentRow['DTDIA'];

                            }
                        }
                    }       
                }
                }
            }
        }
        $return = array(
                        "TABCAL"=>$currentRow['TABCAL'],
                        "newDataSet"=>$newDataSet
                        );
        return $return;
    }

    public function delete($request, $response){
        $this->entityManager->getConnection()->beginTransaction();
        try{    
            $dataSet = $request->getDataset()->getRows()[0];
            $calendarType = $request->getDataset()->getRows()[1];
            $TABS = $request->getDataset()->getRows()[2];
            $info = $request->getDataset()->getRows()[3];
            $tipo = $info[0]["tipo"];
            $CDFILIAL = $info[0]["CDFILIAL"];
            $NRORG = $info[0]['NRORG'];
            $i = 0;
            if($tipo == 'Data Unica'){
                for($i = 0; $i < count($calendarType); $i++){
                    $dia = $dataSet['DATA'];
                    $Row = array();
                    $Row['DATA'] = $dia;
                    if(count($calendarType) == 1){
                        $Row[$TABS[$calendarType[$i]['CALENDAR']]['DTFERI']] = DateUtil::truncateString($dia, 'Y/m/d', 'd/m/Y');
                    } else {
                        $Row[$TABS[$calendarType[$i]]['DTFERI']] = DateUtil::truncateString($dia, 'Y/m/d', 'd/m/Y');
                    }
                    $Row['CDFILIAL'] = $CDFILIAL;
                    $Row['TABCAL'] = $calendarType[$i];
                    $Row['TABS'] = $TABS;
                    $Row['NRORG'] = $NRORG;
                    $Row['__is_new'] = false;
                    $Rows = [$Row];
                    if(count($calendarType) == 1){
                        $dataSetToDelete = new DataSet($TABS[$calendarType[$i]['CALENDAR']]['TABFERI'],$Rows);
                    } else {
                        $dataSetToDelete = new DataSet($TABS[$calendarType[$i]]['TABFERI'],$Rows);
                    }
                    $this->crudService->delete($response,$dataSetToDelete);
                }
                $this->entityManager->getConnection()->commit();
            } else if($tipo == 'Varias Datas'){
                for($i = 0; $i < count($calendarType); $i++){
                    $newDataSet = array();
                    foreach ($dataSet as $dia){
                        $Row = array();
                        $Row['DATA'] = $dia;
                        if(count($calendarType) == 1){
                            $Row[$TABS[$calendarType[$i]['CALENDAR']]['DTFERI']] = DateUtil::truncateString($dia, 'Y/m/d', 'd/m/Y');
                        } else {
                            $Row[$TABS[$calendarType[$i]]['DTFERI']] = DateUtil::truncateString($dia, 'Y/m/d', 'd/m/Y');
                        }
                        $Row['CDFILIAL'] = $CDFILIAL;
                        $Row['TABCAL'] = $calendarType[$i];
                        $Row['TABS'] = $TABS;
                        $Row['NRORG'] = $NRORG;
                        $Row['__is_new'] = false;
                        array_push($newDataSet, $Row);
                    }
                    if(count($calendarType) == 1){
                        $dataSetToDelete = new DataSet($TABS[$calendarType[$i]['CALENDAR']]['TABFERI'],$newDataSet);
                    } else {
                        $dataSetToDelete = new DataSet($TABS[$calendarType[$i]]['TABFERI'],$newDataSet);
                    }
                    $this->crudService->delete($response,$dataSetToDelete);
                }
                $this->entityManager->getConnection()->commit();
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), 500));
            $this->entityManager->rollback();
        }
    }
    
    public function deleteCardapio($request, $response){
        $currentRow = $request->getRow();
        if($currentRow['TYPE']=='industrial'){
            $condition = array(
                array(
                    "columnName" =>'CDFILIAL',
                    "operator" => "=",
                    "value" => $currentRow['CDFILIAL']
                ),
                array(
                    "columnName" =>'DTCARDAPIO',
                    "operator" => "=",
                    "value" => $currentRow['DTCARDAPIO']
                )
            );
            $filterCriteria = new FilterCriteria('cardadia_calendario', $condition);
            $dataSet = $this->dataSourceManager->findBy($filterCriteria);
            $rows = $dataSet->getRows();
            foreach ($rows as $key=>$row) {
                $DATADIA = DateTime::createFromFormat('d/m/Y H:i:s',$row['DTCARDAPIO']);
                $DATAPROC = DateTime::createFromFormat('d/m/Y H:i:s',$currentRow['DTPROCE']);
                if($DATADIA<$DATAPROC){
                    unset($rows[$key]);
                }
            }
            $condition = array(
                array(
                    "columnName" =>'CDFILIAL',
                    "operator" => "=",
                    "value" => $currentRow['CDFILIAL']
                ),
                array(
                    "columnName" =>'NRCARDAPIO',
                    "operator" => "=",
                    "value" => $rows[0]['NRCARDAPIO']
                )
            );
            $filterCriteria = new FilterCriteria('cardadia_calendario', $condition);
            $dataSet = $this->dataSourceManager->findBy($filterCriteria);
            $rows = $dataSet->getRows();
            if(!$rows[0]['NRRETIRADA']) {
                $rowToDelete = $request->getRow();
                $rowToDelete['NRCARDAPIO'] = $row['NRCARDAPIO'];
                $rowToDelete['DATAPROC'] = $rowToDelete['DTPROCE'];
                $rowToDelete['__is_new'] = false;
                $dataSet= $this->crudService->buildDataSetIfRow($request,'cardadia_calendario')->getDataset();
                $this->crudService->delete($response,$dataSet);
            }
            

        } else if($currentRow['TYPE']=='merenda'){

            if($currentRow['CDFILIAL']=$currentRow['CDFILCENTPME']){
                $currentRow['DTPROCE']=$currentRow['DTPROCE'];
                $condition = array(
                    array(
                        "columnName" =>'CDFILIAL',
                        "operator" => "=",
                        "value" => $currentRow['CDFILIAL']
                    ),
                    array(
                        "columnName" =>'DTCARDIAPME',
                        "operator" => "=",
                        "value" => $currentRow['DTCARDIAPME']
                    )
                );
                $filterCriteria = new FilterCriteria('cardiapme_calendario', $condition);
                $dataSet = $this->dataSourceManager->findBy($filterCriteria);
                $rows = $dataSet->getRows();
                foreach ($rows as $key=>$row) {
                    $DATADIA = DateTime::createFromFormat('d/m/Y H:i:s',$row['DTCARDIAPME']);
                    $DATAPROC = DateTime::createFromFormat('d/m/Y H:i:s',$currentRow['DTPROCE']); 
                    if($DATADIA<$DATAPROC){
                        unset($rows[$key]);
                    }
                }
                foreach ($rows as $key=>$row) {
                    $rowToDelete = $request->getRow();
                    $rowToDelete['NRCARDPME'] = $row['NRCARDPME'];
                    $rowToDelete['NRCARDIAPME'] = $row['NRCARDIAPME'];
                    $rowToDelete['__is_new']= false;
                    $condition = array(
                        array(
                            "columnName" =>'CDFILCENTPME',
                            "operator" => "=",
                            "value" => $rowToDelete['CDFILCENTPME']
                        ),
                        array(
                            "columnName" =>'NRCARDPME',
                            "operator" => "=",
                            "value" => $rowToDelete['NRCARDPME']
                        ),
                        array(
                            "columnName" =>'NRCARDIAPME',
                            "operator" => "=",
                            "value" => $rowToDelete['NRCARDIAPME']
                        )
    
                    );
                    $filterCriteria = new FilterCriteria('compcardpme', $condition);
                    $dataSet = $this->dataSourceManager->findBy($filterCriteria);
                    $rows = $dataSet->getRows();
                    $rowToDelete['NRSEQPRACPME'] = $rows[0]['NRSEQPRACPME'];
                    $dataSet= $this->crudService->buildDataSetIfRow($request,'compcardpme')->getDataset();
                    $this->crudService->delete($response,$dataSet);
                }

                unset($rowToDelete['CDFILIAL']);
                $rowToDelete['DATAPROC']=$currentRow['DTPROCE'];
                $dataSet= $this->crudService->buildDataSetIfRow($request,'cardiapme_calendario')->getDataset();
                $this->crudService->delete($response,$dataSet);
            } else{
                $currentRow['DTPROCE']=$currentRow['DTPROCE'];
                $condition = array(
                    array(
                        "columnName" =>'CDFILIAL',
                        "operator" => "=",
                        "value" => $currentRow['CDFILIAL']
                    ),
                    array(
                        "columnName" =>'DTCARDIAPME',
                        "operator" => "=",
                        "value" => $currentRow['DTCARDIAPME']
                    )
                );
                $filterCriteria = new FilterCriteria('cardiapme_calendario', $condition);
                $dataSet = $this->dataSourceManager->findBy($filterCriteria);
                $rows = $dataSet->getRows();
                foreach ($rows as $key=>$row) {
                    $DATADIA = DateTime::createFromFormat('d/m/Y H:i:s',$row['DTCARDIAPME']);
                    $DATAPROC = DateTime::createFromFormat('d/m/Y H:i:s',$currentRow['DTPROCESSA']); 
                    if($DATADIA<$DATAPROC){
                        unset($rows[$key]);
                    }
                }
                foreach ($rows as $key=>$row) {
                    $rowToDelete = $request->getRow();
                    $rowToDelete['NRCARDPME'] = $row['NRCARDPME'];
                    $rowToDelete['NRCARDIAPME'] = $row['NRCARDIAPME'];
                    $rowToDelete['__is_new']= false;
                    $condition = array(
                        array(
                            "columnName" =>'CDFILCENTPME',
                            "operator" => "=",
                            "value" => $rowToDelete['CDFILCENTPME']
                        ),
                        array(
                            "columnName" =>'NRCARDPME',
                            "operator" => "=",
                            "value" => $rowToDelete['NRCARDPME']
                        ),
                        array(
                            "columnName" =>'NRCARDIAPME',
                            "operator" => "=",
                            "value" => $rowToDelete['NRCARDIAPME']
                        )
    
                    );
                    $filterCriteria = new FilterCriteria('compcardpme', $condition);
                    $dataSet = $this->dataSourceManager->findBy($filterCriteria);
                    $rows = $dataSet->getRows();
                    $rowToDelete['NRSEQPRACPME'] = $rows[0]['NRSEQPRACPME'];
                    $dataSet= $this->crudService->buildDataSetIfRow($request,'compcardpme')->getDataset();
                    $this->crudService->delete($response,$dataSet);
                }
                unset($rowToDelete['CDFILCENTPME']);
                $rowToDelete['DATAPROC']=$currentRow['DTPROCE'];
                $dataSet= $this->crudService->buildDataSetIfRow($request,'cardiapme_calendario')->getDataset();
                $this->crudService->delete($response,$dataSet);
            }   
        }
    }
    
}