<?php

namespace Teknisa\PED\Modules\Expedicao\Controllers;

use Teknisa\PED\Modules\Expedicao\Services\exp0800ComissaoVendedorPeriodoService;
use Doctrine\ORM\EntityManager;

use \Teknisa\PED\Libraries\PHPMailer;
use Teknisa\PED\Controllers\TeknisaCrud;
use Teknisa\PED\Controllers\CrudController;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;


class exp0800ComissaoVendedorPeriodoController extends TeknisaCrud {
    protected $dataSourceName = 'mapacomperiodo_widget_completo';
    protected $dataSourceNameToSave = '';
    protected $newCode;
    protected $environment;
    protected $entityManager;
    protected $service;


    public function __construct($crudService, ParameterBag $parameterBag, $newCode, $environment, EntityManager $entityManager, exp0800ComissaoVendedorPeriodoService $service)
    {
        $this->newCode = $newCode;
        $this->environment = $environment;
        $this->parameterBag = $parameterBag;
        $this->entityManager = $entityManager;
        $this->service       = $service;
        parent::__construct($crudService, $parameterBag);
    }

    public function salvarMapaComPeriodo(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->dataSourceName = 'mapacomperiodo';
            parent::save($request, $response);
        } catch (\Exception $e) {
            $response->setError(DTO\Response\Error($e->getMessage()));
        }
    }

    public function deletarMapaComPeriodo($request, DTO\Response $response){
        try {
            $this->dataSourceName = 'mapacomperiodo';
            parent::delete($request,$response);
        } catch (\Exception $e) {
            $response->setError(DTO\Response\Error($e->getMessage()));
        }
    }

    public function apurarComissao(DTO\Request $request, DTO\Response $response) {
        
        $row = $request->getRow();
        
    #begins transaction in preparation for possible rollback needs
        $this->entityManager->beginTransaction();
        try {
            #calls parent saving method
            $this->service->deleteMapas($row);
            $row = $this->service->getPoliticaFilial($row);
            
            $rowsToSaveNota = $this->service->getRowsToSaveNota($row);
            
            $this->dataSourceName = 'mapacomexp';
            $dataSource = $this->dataSourceName;
            
            $dataSet = new DataSource\DataSet ($dataSource, $rowsToSaveNota);
            
            $requestSalvar = new DTO\Request\DataSet($dataSet, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
            
            parent::save($requestSalvar, $response);
            
            $rowsToSaveContas = $this->service->getRowsToSaveContas($row);
            $this->dataSourceName = 'mapacomrec';
            $dataSource = $this->dataSourceName;
           
            $dataSetContas = new DataSource\DataSet ($dataSource, $rowsToSaveContas);
               
            $requestSave = new DTO\Request\DataSet($dataSetContas, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
            
            parent::save($requestSave, $response);
            
            #commit successful transaction*/
            $this->entityManager->commit();
        } catch(\Exception $e) {
            #rollbacks previous operations
            $this->entityManager->rollback();
            
            #catches error and sets message as needed
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }

    public function saveNotaForaPeriodo(DTO\Request $request, DTO\Response $response) {
        try{
            $this->dataSourceName = 'mapacomexp';
            $dataSource = $this->dataSourceName;
            $value = $request->getRow();
            $row = array();
            $rowsToSave = null;
            unset($value['__is_new']);
            $row['CDFILIAL']    = $value['CDFILIAL'];
            $row['VENDEDOR']    = $value['CDVENDEDOR'];
            $row['NRORG']       = $value['NRORG'];
            $row['NRLANCTONF']  = $value['NRLANCTONF'];
            $row['NRMAPACOMIS'] = $value['NRMAPACOMIS'];
            $row = $this->service->getPoliticaFilial($row);
            $rowsToSave = $this->service->getRowToSaveNota($value, $row);
           if(isset($rowsToSave['OUTROPERIODO'])) {
                if($rowsToSave['OUTROPERIODO'] === 'S') {
                    $erro = array("ERRO" => true);
                    $dataSetContas = new DataSource\DataSet ($dataSource, $erro);
                    $response->addDataSet($dataSet);
                }
                else {
                    $requestSalvar = new DTO\Request\Row($rowsToSave, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
                    parent::save($requestSalvar, $response);
                }    
            }
            else {
                    $requestSalvar = new DTO\Request\Row($rowsToSave, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
                    parent::save($requestSalvar, $response);
            } 
        }
        catch(\Exception $e) {
              $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }
    
    public function saveContaForaPeriodo(DTO\Request $request, DTO\Response $response) {
        try {
            $this->dataSourceName = 'mapacomrec';
            $dataSource = $this->dataSourceName;
            $value = $request->getRow();
            $row                = array();
            $row['CDFILIAL']    = $value['CDFILIAL'];
            $row['VENDEDOR']    = $value['CDVENDEDOR'];
            $row['NRORG']       = $value['NRORG'];
            $row['NRLANCTONF']  = $value['NRLANCTONF'];
            $row['NRMAPACOMIS'] = $value['NRMAPACOMIS'];
            $row = $this->service->getPoliticaFilial($row);
            $rowsToSave = array();
            unset($value['__is_new']);
            $rowsToSave = $this->service->getRowToSaveConta($value, $row);
            
            if(isset($rowsToSave)){
                if(!isset($value['NEWROW'])) {
                    $dataSet = new DataSource\DataSet('mapacomrec', $rowsToSave);
                }
                if(isset($rowsToSave['OUTROPERIODO'])) {
                    if($rowsToSave['OUTROPERIODO'] === 'S') {
                        $erro = array("ERRO" => true);
                        $dataSetContas = new DataSource\DataSet ($dataSource, $erro);
                        $response->addDataSet($dataSet);
                    }
                    else {
                        if(isset($value['NEWROW'])) {
                           $requestSalvar = new DTO\Request\Row($rowsToSave, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
                        }
                        else {
                           $requestSalvar = new DTO\Request\DataSet($dataSet, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
                        }
                        parent::save($requestSalvar, $response);
                    }    
                }
                else {
                       if(isset($value['NEWROW'])) {
                           $requestSalvar = new DTO\Request\Row($rowsToSave, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
                        }
                        else {
                           $requestSalvar = new DTO\Request\DataSet($dataSet, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
                        }
                        parent::save($requestSalvar, $response);
                }    
            }
            else {
                throw new \Exception('Error in data save');
            }
        }
        catch(\Exception $e) {
              $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }
    
    public function verificaAlertaMapas(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->service->verificaAlertaMapas($row);
            $dataSource = 'mapacomexp';
            $rowReturn = $this->service->verificaAlertaMapas($row);
            
            $dataSet = new DataSource\DataSet ($dataSource, $rowReturn);
            $response->addDataSet($dataSet);
        }
        catch(\Exception $e) {
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }
    
    public function findProdutosNotaFiscal(DTO\Request\Filter $request, DTO\Response $response) {
        try{ 
            $dataSource = 'produtosNotaFiscal';
            $conditions = $request->getFilterCriteria()->getConditions();
            $row = array();
            $count = 0;
            foreach($conditions as $condition) {
                $row[$condition['columnName']] = $condition['value'];
                
            }
            $rowToResponse = $this->service->getProdutosNotaFiscal($row);
            $dataSet = new DataSource\DataSet ($dataSource, $rowToResponse);
            $response->addDataSet($dataSet);
        }
        catch(\Exception $e) {
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }
    
    public function gerarContaPagar(DTO\Request $request, DTO\Response $response) {
        $this->dataSourceName = 'mapacompag';
        $this->entityManager->beginTransaction();
        try {
            $row           = $request->getRow();
            $rowsToSave     = $this->service->gerarContaPagar($row); 
            $dataSet = new DataSource\DataSet('mapacompag', $rowsToSave);
            $requestSalvar = new DTO\Request\DataSet($dataSet, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
            parent::save($requestSalvar, $response);
            $this->entityManager->commit();
        }
        catch(\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function saveComissaoProduto(DTO\Request $request, DTO\Response $response) {
        $row        = $request->getRow();
        $this->dataSourceName = 'venditemped';
        $this->entityManager->beginTransaction();
        try{
            $notasReapurar = $this->service->getNotasReapurar($row);
            if($notasReapurar === false) {
                throw new \Exception('Existe uma conta a pagar ligada a este produto. Não foi possivel alterar a comissão');
            }
            else {
                $row['__is_new'] = false;
                $requestSalvar = new DTO\Request\Row($row, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
                parent::save($requestSalvar, $response);
                $notasToSave  = array();
                $contasToSave = array();
                $value        = array();
                $value['CDFILIAL']    = $row['CDFILIAL'];
                $value['VENDEDOR']    = $row['CDVENDEDOR'];
                $value['NRORG']       = $row['NRORG'];
                $value['NRMAPACOMIS'] = $row['NRMAPACOMIS'];
                $value['__is_new']    = false;
                $value = $this->service->getPoliticaFilial($row);
                if(isset($notasReapurar)) {
                    foreach($notasReapurar as $nota) {
                        $value['NRLANCTONF']    = $nota['NRLANCTONF'];
                        $notaSave['__is_new']   = false;
                        $notaSave['NRLANCTONF'] = $nota['NRLANCTONF'];
                        $notaSave['CDFILICOMIS']= $value['CDFILIAL'];
                        $notaSave['CDVENDEDOR'] = $row['CDVENDEDOR'];
                        $rowUpdate              =  $this->service->getNRMAPACOMEXP($notaSave);
                        $notaSave               = $this->service->getRowToSaveNota($value, $row);
                        if(isset($rowUpdate[0]['NRMAPACOMEXP'])) {
                            $notaSave['NRMAPACOMEXP'] = $rowUpdate[0]['NRMAPACOMEXP'];
                            if(isset($notaSave['NRLANCTONF'])) {
                                $notaSave['__is_new']   = false;
                                $notaSave['NRORG']      = $row['NRORG'];
                                $notaSave['NRMAPACOMIS'] = $rowUpdate[0]['NRMAPACOMIS'];
                                array_push($notasToSave, $notaSave);
                            }
                        }
                        $contasNota = $this->service->getContasNota($value);
                        foreach ($contasNota as $conta) {
                            $row['NRSEQVENREC']       = $conta['NRSEQVENREC'];
                            $row['VRATUAREC']         = $conta['VRATUAREC'];
                            $conta['__is_new']        = false; 
                            $contaSave['NRORG']       = $row['NRORG'];
                            $contaSave['NRLANCTONF']  = $nota['NRLANCTONF'];
                            $contaSave['CDFILICOMIS'] = $value['CDFILIAL'];
                            $contaSave['NRSEQVENREC'] = $conta['NRSEQVENREC'];
                            $contaSave['CDVENDEDOR']  = $row['CDVENDEDOR'];
                            $rowUpdate                = $this->service->getNRMAPACOMREC($contaSave);
                            
                            if(isset($rowUpdate[0]['NRMAPACOMREC'])) {
                                $contaSave                 = $this->service->getRowToSaveConta($conta, $row);
                                $contaSave['NRMAPACOMREC'] = $rowUpdate[0]['NRMAPACOMREC'];
                                if(isset($contaSave['NRLANCTONF'])) {
                                    $contaSave['__is_new']    = false;
                                    $contaSave['CDFILICOMIS'] = $value['CDFILIAL'];
                                    $contaSave['CDVENDEDOR']  = $value['CDVENDEDOR'];
                                    $contaSave['NRMAPACOMIS'] = $rowUpdate[0]['NRMAPACOMIS'];
                                    array_push($contasToSave, $contaSave);
                                }
                            }
               
                        }
                    }
                }
                if(!(empty($notasToSave))) {
                    $dataSetNotas  = new DataSource\DataSet('mapacomexp', $notasToSave );
                    $this->dataSourceName = 'mapacomexp';
                    $requestSalvar = new DTO\Request\DataSet($dataSetNotas, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
                    parent::save($requestSalvar, $response);
                }
                if(!(empty($contasToSave))) {
                    $dataSetContas = new DataSource\DataSet('mapacomrec', $contasToSave);
                    $this->dataSourceName = 'mapacomrec';
                    $requestSalvar = new DTO\Request\DataSet($dataSetContas, $request->getMethod(),$request->getRoutePath(), $request->getUserId());
                    parent::save($requestSalvar, $response);
                }
                
            }
            $this->entityManager->commit();
        }
        catch(\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }
    
    public function excluirMapa(DTO\Request $request, DTO\Response $response) {
        $row        = $request->getRow();
        $row['__is_new'] = false;
        $this->entityManager->beginTransaction();
        try{
           
            if($row['dataSourceName'] === 'mapacomperiodo') {
                $conditions = array(
        	        array(
            	        "value" => $row['NRMAPACOMIS'],
            	        "operator" => "=",
            	        "columnName" => "NRMAPACOMIS"
            	      ),
        	       array(
        	            "value" => $row['CDVENDEDOR'],
        	            "operator" => "=",
            	        "columnName" => "CDVENDEDOR"
            	      ),
            	   array(
            	        "value" => $row['CDFILICOMIS'],
        	            "operator" => "=",
            	        "columnName" => "CDFILICOMIS"
            	   ),
            	   array(
            	        "value" => $row['NRORG'],
        	            "operator" => "=",
            	        "columnName" => "NRORG"
            	   )
    	        );
    	        
    	        $this->dataSourceName = 'mapacomexp';
                $filterCriteria = new FilterCriteria('mapacomexp', $conditions);
                $this->crudService->find($response, $filterCriteria);
                $dataSet = new DataSource\DataSet('mapacomexp', $response->getDataSets()[0]->getRows());
                $this->crudService->delete($response, $dataSet);
                
                $this->dataSourceName = 'mapacomrec';
                $filterCriteria = new FilterCriteria('mapacomrec', $conditions);
                $this->crudService->find($response, $filterCriteria);
                $dataSet = new DataSource\DataSet('mapacomrec', $response->getDataSets()[1]->getRows());
                $this->crudService->delete($response, $dataSet);
                //Deleta Registros filhos
            }
            $rowDelete = array($row);
            $this->dataSourceName = $row['dataSourceName'];
            $dataSet = new DataSource\DataSet('mapacomperiodo', $rowDelete);
            $this->crudService->delete($response, $dataSet);
            $this->entityManager->commit();
        }
        catch(\Exception $e) {
            $this->entityManager->rollback();
            $response->setError(new Error("Error in data save. ".$e->getMessage(), 500));
        }
    }
    
    //-----------------------------------------------------------------------------------------------------//
    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
}
